<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if(!class_exists('FPD_Admin_Ajax')) {

	class FPD_Admin_Ajax {

		public function __construct() {

			//react-app

			// -products
			add_action( 'wp_ajax_fpd_get_products', array( &$this, 'get_products' ) );
			add_action( 'wp_ajax_fpd_create_product', array( &$this, 'create_product' ) );
			add_action( 'wp_ajax_fpd_update_product', array( &$this, 'update_product' ) );
			add_action( 'wp_ajax_fpd_delete_product', array( &$this, 'delete_product' ) );

			// -product categories
			add_action( 'wp_ajax_fpd_get_product_categories', array( &$this, 'get_product_categories' ) );
			add_action( 'wp_ajax_fpd_create_product_category', array( &$this, 'create_product_category' ) );
			add_action( 'wp_ajax_fpd_update_product_category', array( &$this, 'update_product_category' ) );
			add_action( 'wp_ajax_fpd_delete_product_category', array( &$this, 'delete_product_category' ) );

			// - templates
			add_action( 'wp_ajax_fpd_get_templates', array( &$this, 'get_templates' ) );
			add_action( 'wp_ajax_fpd_create_template', array( &$this, 'create_template' ) );
			add_action( 'wp_ajax_fpd_delete_template', array( &$this, 'delete_template' ) );

			// - views
			add_action( 'wp_ajax_fpd_get_view', array( &$this, 'get_view' ) );
			add_action( 'wp_ajax_fpd_update_view', array( &$this, 'update_view' ) );
			add_action( 'wp_ajax_fpd_delete_view', array( &$this, 'delete_view' ) );

			// - options
			add_action( 'wp_ajax_fpd_get_options', array( &$this, 'get_options' ) );
			add_action( 'wp_ajax_fpd_update_options', array( &$this, 'update_options' ) );

			// - UI composer
			add_action( 'wp_ajax_fpd_get_colors_css', array( &$this, 'get_colors_css' ) );
			add_action( 'wp_ajax_fpd_get_ui_layouts', array( &$this, 'get_ui_layouts' ) );
			add_action( 'wp_ajax_fpd_get_ui_layout', array( &$this, 'get_ui_layout' ) );
			add_action( 'wp_ajax_fpd_create_ui_layout', array( &$this, 'create_ui_layout' ) );
			add_action( 'wp_ajax_fpd_update_ui_layout', array( &$this, 'update_ui_layout' ) );
			add_action( 'wp_ajax_fpd_delete_ui_layout', array( &$this, 'delete_ui_layout' ) );

			// - design categories
			add_action( 'wp_ajax_fpd_get_design_categories', array( &$this, 'get_design_categories' ) );
			add_action( 'wp_ajax_fpd_get_design_category', array( &$this, 'get_design_category' ) );
			add_action( 'wp_ajax_fpd_create_design_category', array( &$this, 'create_design_category' ) );
			add_action( 'wp_ajax_fpd_update_design_category', array( &$this, 'update_design_category' ) );
			add_action( 'wp_ajax_fpd_delete_design_category', array( &$this, 'delete_design_category' ) );

			// - options
			add_action( 'wp_ajax_fpd_get_orders', array( &$this, 'get_orders' ) );
			add_action( 'wp_ajax_fpd_get_order', array( &$this, 'get_order' ) );
			add_action( 'wp_ajax_fpd_update_order', array( &$this, 'update_order' ) );
			add_action( 'wp_ajax_fpd_delete_order', array( &$this, 'delete_order' ) );

			// - orders
			add_action( 'wp_ajax_fpd_create_file_export', array( &$this, 'create_file_export' ) );

		}

		public function get_products() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$defaults = array(
				'include_views' => true,
				'page' => 1,
				'limit' => 20,
				'search' => null,
				'category_id' => null,
				'cols' => '*',
				'filter_by' => 'ID',
				'sort_by' => 'ASC',
				'user_id' => null
			);

			$args = wp_parse_args( $_GET, $defaults );

			$response = FPD_Resource_Products::get_products( $args );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function create_product() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Products::create_product( $payload );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function update_product() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Products::update_product( $payload['id'], $payload['data'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function delete_product() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Products::delete_product( $payload['id'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function get_product_categories() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$response = FPD_Resource_Products::get_all_product_categories();

			echo json_encode( $response );

			die;

		}

		public function create_product_category() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Products::create_product_category( $payload );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function update_product_category() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Products::update_product_category( $payload['id'], $payload['data'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function delete_product_category() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Products::delete_product_category( $payload['id'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function get_templates() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$defaults = array(
				'type' => 'user'
			);

			$args = wp_parse_args( $_GET, $defaults );

			$response = FPD_Resource_Templates::get_product_templates( $args );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function create_template() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Templates::create_product_template( $payload );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function delete_template() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Templates::delete_product_template( $payload['id'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function get_view() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$response = FPD_Resource_Views::get_product_view( $_GET['id'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function update_view() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Views::update_product_view( $payload['id'], $payload['data'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function delete_view() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Views::delete_product_view( $payload['id'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function get_options() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$option_keys = isset( $_GET['keys'] ) ? $_GET['keys'] : array();

			$response = FPD_Resource_Options::get_options( $option_keys );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function update_options() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			if( is_array($payload) ) {

				foreach($payload as $key => $value) {
					update_option( $key, $value );
				}

			}

			echo json_encode($payload);

			die;

		}

		public function get_colors_css() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			if( isset($payload['primary_color']) ) {

				$response = self::parse_css($payload['primary_color'], $payload['secondary_color']);

				if(is_wp_error( $response )) {
					wp_send_json_error($response->get_error_message(), 500);
				}
				else {
					echo $response;
				}

			}

			die;

		}

		public function get_ui_layouts() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$response = FPD_Resource_UI_Layouts::get_ui_layouts();

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function get_ui_layout() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$response = FPD_Resource_UI_Layouts::get_ui_layout( $_GET['id'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function create_ui_layout() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_UI_Layouts::create_ui_layout( $payload );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function update_ui_layout() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_UI_Layouts::update_ui_layout( $payload['id'], $payload['data'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function delete_ui_layout() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_UI_Layouts::delete_ui_layout( $payload['id'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function get_design_categories() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$response = FPD_Resource_Designs::get_design_categories();

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function get_design_category() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$response = FPD_Resource_Designs::get_single_category( $_GET['id'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function create_design_category() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Designs::create_design_category( $payload['title'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function update_design_category() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Designs::update_design_category( $payload['id'], $payload['data'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function delete_design_category() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Designs::delete_design_category( $payload['id'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function get_orders() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$defaults = array(
				'page' => 1,
				'type' => 'shortcode',
			);

			$args = wp_parse_args( $_GET, $defaults );

			$response = FPD_Resource_Orders::get_all_orders( $args );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function get_order() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$response = FPD_Resource_Orders::get_single_order( $_GET );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function update_order() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Orders::update_order( $payload['id'], $payload['data'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function delete_order() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = FPD_Resource_Orders::delete_order( $payload['id'] );

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( $response );
			}

			die;

		}

		public function create_file_export() {

			check_ajax_referer( 'fpd_ajax_nonce' );

			$payload = json_decode( file_get_contents('php://input'), true );

			$response = $file_url = null;

			if( isset($payload['summary_json']) && is_string($payload['summary_json']) )
				$payload['summary_json'] = json_decode( stripslashes($payload['summary_json']), true );

			if( isset($payload['print_ready']) && $payload['print_ready'] ) {

				if( class_exists('Fancy_Product_Designer_Export') ) {

					//var_dump($payload);
					$file = Fancy_Product_Designer_Export::create_print_ready_file( $payload );
					$file_url = content_url( '/fancy_products_orders/print_ready_files/' . $file );

				}
				else
					$response = new WP_Error(
						'print-ready-export-not-installed',
						__('The Print-Ready add-on is not installed', 'radykal')
					);

			}
			else if( isset($payload['svg_data']) ) {
				$file_url = FPD_File_Export::svg_to_pdf( $payload );
			}

			if(is_wp_error( $response )) {
				wp_send_json_error($response->get_error_message(), 500);
			}
			else {
				echo json_encode( array(
					'url' => $file_url
				) );
			}

			die;

		}

		public static function parse_css( $primary_color='', $secondary_color='') {

			$result = FPD_UI_Layout_Composer::parse_css('@primaryColor: '.$primary_color.'; @secondaryColor: '.$secondary_color.';');

			if( !is_array($result) ) {

				return json_encode(array(
					'css' => $result
				));

			}
			else {

				return new WP_Error(
					'parse-colors-css-fail',
					$result['message']
				);

			}

		}

	}
}

new FPD_Admin_Ajax();

?>